#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)

import sys
import logging
import os
import subprocess
from datetime import datetime
from cdo import Cdo

# Local import
from utils.utils_files import *

"""
Class: Temporal completeness checks
---
- Uses CDO python bindings
- Loops over all timestamps within the dataset
- Checks they are within the time range [start/end] defined as input
"""
class temporal_completeness_test:

    def __init__(self, logdir, path, pattern):
        self.testid = 'test03-temporal-completeness'
        self.logdir = logdir 
        self.path = path        
        self.files = dir_or_file(path, pattern)
        # Compute start and end date using file names
        self.tstart_dt, self.tend_dt = get_t0_tn(self.files)
        
    def check_timestamps(self):

        # For all files
        cdo = Cdo()
        res = True
        
        # Progress init
        i = 0
        n = len(self.files)
        print()

        for f in self.files:
            # Progress update
            i+=1
            print_progress(i, n)                          
            logging.info('Checking {}'.format(f))

            try:
                info = cdo.showtimestamp(input=f)
            except:
                logging.error('Bad file format (cdo cannot read file)')
                print('ERROR: Bad file format ({})'.format(os.path.basename(f)))
                res = False
                break

            for inf in info:
                if not res: break # no need to proceed
                for t in inf.split():                    
                    date = datetime.strptime(t, '%Y-%m-%dT%H:%M:%S')
                    if not(date >= self.tstart_dt and date <= self.tend_dt):
                        logging.error('Date out of range {} [{} -> {}]'.format(t, self.tstart_dt, self.tend_dt))
                        res = False                        

        # Final message
        if res:
            logging.info('FILES ARE TEMPORALLY COMPLETE')
            print('INFO: FILES ARE TEMPORALLY COMPLETE')
        else:
            logging.error('TEMPORAL INCOMPLETENESS FOUND')
            print('INFO: TEMPORAL INCOMPLETENESS FOUND')

    def main(self):
        # Initiate the appropiate logs
    	initiate_logging(self.logdir, self.path, self.testid)

        # Do the actual work
    	self.check_timestamps()
                
if __name__ == "__main__":
    sys.exit()
